<?php

/*//. KatePKG - A GUI (GTK2) Package Manger for KateOS.
Written by Bob Majdak Jr <bob@kateos.org>.

This program is released under the GNU GPL.

See $PREFIX/bin/katepkg for details .//*/

class packageTextView extends GtkScrolledWindow {

	var $text;

	function packageTextView() {
		parent::__construct();

		$this->view = new GtkTextView;
		$this->buffer = new GtkTextBuffer;
		$this->empty = new GtkTextBuffer;

		$this->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
		$this->set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);
		$this->view->set_wrap_mode(2);

		$this->view->set_editable(0);
		$this->view->set_cursor_visible(0);
		$this->view->set_border_width(2);
		$this->view->set_buffer($this->buffer);	

		//. build tags we will use.		
		$this->tag_bold = new GtkTextTag("bold");
		$this->tag_bold->set_property("weight","700");

		//. get table, append tags.
		$this->tag_table = $this->buffer->get_tag_table();
		$this->tag_table->add($this->tag_bold);
		
		$this->add($this->view);	
		
		$this->text = "";
		
		$this->show_all();
	}
	
	function append_text($in) {
	
		$arg = func_get_args();	
	
		//. scan incoming text for formatting.
		while(preg_match("/<b>.*?<\/b>/ms",$in)) {
			$bold_startchar[] = strpos(utf8_decode($in),"<b>") + strlen(utf8_decode($this->text));
			$bold_stopchar[] = strpos(utf8_decode($in),"</b>") + strlen(utf8_decode($this->text)) - 3;
			$in = preg_replace("/<\/?b>/","",$in,2);
		}
		
		//. record and append clean text.
		$this->text .= $in;		
		$this->buffer->insert_at_cursor($in,strlen($in));
		//$this->buffer->set_text($this->text);

		//. build mark points for bolding.
		for($a = 0; $a < count($bold_startchar); $a++) {
			$this->buffer->apply_tag(
				$this->tag_bold,
				$this->buffer->get_iter_at_mark($this->buffer->create_mark($b++, $this->buffer->get_iter_at_offset($bold_startchar[$a]), FALSE)),
				$this->buffer->get_iter_at_mark($this->buffer->create_mark($b++, $this->buffer->get_iter_at_offset($bold_stopchar[$a]), FALSE))			
			);
		}

		//. scroll to bottom of window.
		if($arg[1] == TRUE) {
			$this->view->scroll_to_mark($this->buffer->get_insert(),0);
		}
		
	
	}
	
	function scroll_down() {
		$this->view->scroll_to_mark($this->buffer->get_insert(),0);
	}
	
	function clear_text() {
		$this->text = "";
		$this->buffer->set_text("");
	}

}

class packageListView extends GtkScrolledWindow {
	
	var $clicked_once;	
	
	function packageListView() {
		parent::__construct();

		//. build subwidgets.
		$this->empty = new GtkListStore(KATE_TYPE_STRING);
		$this->store = new GtkListStore(KATE_TYPE_STRING,KATE_TYPE_STRING);
		$this->view = new GtkTreeView;
		$this->c1 = new GtkTreeViewColumn(_("Package"),new GtkCellRendererText,'text',0);
		$this->c2 = new GtkTreeViewColumn(_("Description"),new GtkCellRendererText,'text',1);

		//. configure oneself.
		$this->set_policy(Gtk::POLICY_AUTOMATIC,GTK::POLICY_AUTOMATIC);
		$this->c1->set_sort_column_id(0);
		
		$this->view->append_column($this->c1);
		$this->view->append_column($this->c2);	
		$this->view->set_expander_column($this->c2);
		$this->view->set_model($this->store);
		
		//. compile oneself.
		$this->add($this->view);	
		
		$this->clicked_once = FALSE;
		
	}	
	
}

class packageWindow extends GtkWindow {

	var $basetitle;	
	
	function packageWindow() {
		parent::__construct();
		
		global $config;
		global $defaultconfig;

		$this->basetitle = "KatePKG";

		$this->set_title($this->basetitle);
		$this->set_size_request($defaultconfig['katepkg_width'],$defaultconfig['katepkg_height']);
		$this->set_default_size($config['katepkg_width'],$config['katepkg_height']);
		$this->set_position(Gtk::WIN_POS_CENTER);
		
		$this->connect_simple("size-request","katepkg_window_resize");		
		$this->connect_simple("delete-event","package_quit");
		
		$this->ui = new packageMainUI;
		
		$this->add($this->ui);
		
		$this->show_all();
		
	}
	
	function update_title($string) {
		if($string) {
			$this->set_title((sprintf("%s - %s",$this->basetitle,$string)));
		} else {
			$this->set_title($this->basetitle);
		}

//		package_run_ui();

	}	
	
}

class packageMainUI extends GtkVbox {

	var $opencmd;

	function packageMainUI() {
		parent::__construct();
		
		global $config;

		//. create main ui widgets.
		$this->menu_area = new GtkMenuBar;
		$this->main_area = new GtkHBox;
		$this->mode_area = new GtkNotebook;
		$this->mode_browse = new packageViewBrowse;
		$this->mode_update = new packageViewUpdate;
		$this->mode_install = new packageViewInstall;
		$this->status = new GtkStatusBar;
		$this->progress = new GtkProgressBar;
		
		//. base config widgets.
		$this->mode_area->set_tab_pos(Gtk::POS_BOTTOM);
		$this->mode_area->connect_after('switch-page','package_page_select');
		
		$this->status->set_has_resize_grip(TRUE);
		
		$this->progress->set_size_request(100,PENDING);
		$this->progress->set_pulse_step(0.01);
		
		//. define the menu system.
		$this->define_menu = array(
			'package' => array('menu',_('Package')),
			'package_browse' => array('item',_('Browse Installed Packages'),'package_page_flip:0','package'),
			'package_update' => array('item',_('Update Installed Packages'),'package_page_flip:1','package'),
			'package_install' => array('item',_('Install New Package'),'package_page_flip:2','package'),
			'hr1' => array('hr','package'),
			'edit_repo' => array('item',_('Edit Repository List'),'katepkg_srcedit_dialog','package'),
			'open_download_folder' => array('item',_('Open Download Folder'),'katepkg_open_download_directory','package'),
			'refresh_repo' => array('item',_('Sync with Repositories Now'),'package_fetch_updatelist','package'),
			'hr2' => array('hr','package'),
			'startup_repo_sync' => array('option',_('Sync Repositories on Startup'),'package_config_toggle:startup_repo_sync','package'),
			'auto_close_update_dialog' => array('option',_('Close Repository Sync when Done'),'package_config_toggle:auto_close_update_dialog','package'),
//			'auto_refresh_local_list' => array('option','Reload Local List After Update/Install','package_config_toggle:auto_refresh_local_list','package'),
			'hr3' => array('hr','package'),
			'track_deps' => array('option',_('Check Dependencies on Download'),'package_config_toggle:track_deps','package'),
			'download_errors_fatal' => array('option',_('Download Errors Prevent Install/Update'),'package_config_toggle:download_errors_fatal','package'),
			'keep_downloaded_packages' => array('option',_('Keep Downloaded Packages'),'package_config_toggle:keep_downloaded_packages','package'),
//			'download_packages_first' => array('option','Download All Before Install/Update','package_config_toggle:download_packages_first','package'),
			'download_without_install' => array('option',_('Download Without Install/Update'),'package_config_toggle:download_without_install','package'),
			'hr4' => array('hr','package'),
			'debug' => array('option',_('Enable Debugging Output'),'package_config_toggle:debug','package'),			
			'hr5' => array('hr','package'),
			'quit' => array('item',_('Quit'),'package_quit','package'),
			'help' => array('menu',_('Help')),
			'about' => array('item',_('About KatePKG'),'katepkg_dialog_about','help')
		);
		
		//. detect if we have any file browser utilities.
		//. If the user has gnome-open they might be using something other than thunar.
		if(`which gnome-open 2> /dev/null`) {
			$this->opencmd = "gnome-open %s &> /dev/null &";
		}
		
		//. else - KateOS uses Thunar.
		else if(`which thunar 2> /dev/null`) {
			$this->opencmd = "thunar %s &> /dev/null &";
		}
		
		//. else get rid of this option.
		else {
			$this->opencmd = FALSE;
			unset($this->define_menu['open_download_folder']);
		}
		
		//. build menus.
		foreach($this->define_menu as $id => $item) {
			switch($item[0]) {
				case('menu'): {
					$this->menu[$id] = new GtkMenu;
					$this->menuitem[$id] = new GtkMenuItem($item[1]);
					$this->menuitem[$id]->set_submenu($this->menu[$id]);
					$this->menu_area->add($this->menuitem[$id]);
					break;
				}
				case('item'): {
					$callback = explode(":",$item[2]);
					$this->menuitem[$id] = new GtkMenuItem($item[1]);
					$this->menuitem[$id]->connect_simple('activate',$callback[0],$callback[1]);					
					$this->menu[$item[3]]->add($this->menuitem[$id]);
					break;
				}
				case('option'): {
					$callback = explode(":",$item[2]);
					$this->menuitem[$id] = new GtkMenuItem();
					$this->menuitem[$id]->connect_simple('activate',$callback[0],$callback[1]);
					
					//. this fixes the weird gtk2 highlight - better solution is GtkCheckMenuItem but it Segfaults when
					//. switching activation state.
					$this->menuoption[$id] = new GtkCheckButton;
					$this->menuoptionlabel[$id] = new GtkLabel($item[1]);
					$this->menuoptionbox[$id] = new GtkHBox;
					$this->menuoptionbox[$id]->pack_start($this->menuoption[$id],FALSE,FALSE);
					$this->menuoptionbox[$id]->pack_start($this->menuoptionlabel[$id],FALSE,FALSE);
					$this->menuitem[$id]->add($this->menuoptionbox[$id]);
					//. / ../
									
					$this->menu[$item[3]]->add($this->menuitem[$id]);
					break;
				}
				case('hr'): {
					$this->menuitem[$id] = new GtkSeparatorMenuItem();
					$this->menu[$item[1]]->add($this->menuitem[$id]);
					break;
				}
			}
		} 

		//. set main ui icons.
		$this->mode_browse->refresh->set_image(katepkg_icon('refresh'));
		$this->mode_browse->uninstall->set_image(katepkg_icon('uninstall'));
		
		$this->mode_update->refresh->set_image(katepkg_icon('refresh'));
		$this->mode_update->update->set_image(katepkg_icon('update'));
		$this->mode_update->browse->set_image(katepkg_icon('browse'));
		$this->mode_update->distupdate->set_image(katepkg_icon('distupdate'));		
		
		$this->mode_install->refresh->set_image(katepkg_icon('refresh'));
		$this->mode_install->install->set_image(katepkg_icon('install'));
		$this->mode_install->browse->set_image(katepkg_icon('browse'));
		$this->mode_install->dirinstall->set_image(katepkg_icon('installdir'));
		
		//. move the bars into position.
		$this->mode_browse->pane->set_position($config['katepkg_barpos']);
		$this->mode_update->pane->set_position($config['katepkg_barpos']);
		$this->mode_install->pane->set_position($config['katepkg_barpos']);
		
		//. final main ui packing.
		$this->mode_area->append_page($this->mode_browse);
		$this->mode_area->set_tab_label_text($this->mode_browse,_("Browse Installed"));
		$this->mode_area->append_page($this->mode_update);
		$this->mode_area->set_tab_label_text($this->mode_update,_("Update System"));
		$this->mode_area->append_page($this->mode_install);
		$this->mode_area->set_tab_label_text($this->mode_install,_("Install Packages"));
		
		$this->main_area->pack_start($this->mode_area,TRUE,TRUE);
	
		$this->status->pack_start($this->progress,FALSE,FALSE,0);		
		
		$this->pack_start($this->menu_area,FALSE,FALSE);
		$this->pack_start($this->main_area,TRUE,TRUE);
		$this->pack_start($this->status,FALSE,FALSE);
		
	}
	
	//. every gtk theme i have ever used, (including windows) has had crap padding on the
	//. status bar text. so i am going to pad a space and see if anyone cares.	
	
	function status_text($text) {
		$this->status->pop(0);
		$this->status->push(0," ".$text);
	}
	
	function status_text_timeout($text) {
		if($this->timeout) { gtk::timeout_remove($this->timeout); }
		$this->status->pop(1);
		$this->status->push(1," ".$text);
		$this->timeout = gtk::timeout_add(2000,array($this,'status_timeout_clear'));
	}
	
	function status_timeout_clear() {
		$this->status->pop(1);//. setting the base statusbar help for this tab.
	}

}

class packageViewBrowse extends GtkVBox {	

	function packageViewBrowse() {
		parent::__construct();
		
		//. build view widgets.
		$this->pane = new GtkVPaned;
		$this->list = new packageListView;
		$this->info = new packageTextView;
		$this->row_buttons = new GtkHBox;
		$this->bottom = new GtkVBox;
		$this->uninstall = new GtkButton(_("Uninstall Selected"));
		$this->refresh = new GtkButton(_('Refresh List'));
		
		//. configure view.
		$this->pane->set_position(200);
		
		$this->selection = $this->list->view->get_selection();
		$this->selection->connect("changed",'package_view_details');
//		$this->selection->connect("double-click-event",'package_dialog_uninstall_check');
		$this->selection->set_mode(Gtk::SELECTION_MULTIPLE);

		$this->uninstall->set_sensitive(FALSE);		
		$this->uninstall->connect_simple('clicked','package_dialog_uninstall_check');
		
		$this->refresh->connect_simple('clicked','package_list_installed');
		
		//. compile view.

//		$this->row_buttons->pack_end($this->update,FALSE,FALSE);
		$this->row_buttons->pack_end($this->uninstall,FALSE,FALSE);
		$this->row_buttons->pack_end($this->refresh,FALSE,FALSE);
		$this->bottom->pack_start($this->info,TRUE,TRUE);
		$this->bottom->pack_start($this->row_buttons,FALSE,FALSE);
		$this->pane->add1($this->list);
		$this->pane->add2($this->bottom);
		
		
		//. final build.
		$this->pack_start($this->pane,TRUE,TRUE);
	}
	
}

class packageViewUpdate extends GtkVbox {

	function packageViewUpdate() {
		parent::__construct();

		//. build view widgets.
		$this->pane = new GtkVPaned;
		$this->list = new packageListView;
		$this->info = new packageTextView;
		$this->row_buttons = new GtkHBox;
		$this->bottom = new GtkVBox;
		$this->update = new GtkButton(_("Update Selected"));
		$this->refresh = new GtkButton(_("Refresh List"));
		$this->browse = new GtkButton(_('Browse...'));
		$this->distupdate = new GtkButton(_('Distribution Update'));	
		
		//. configure view.
		$this->pane->set_position(200);
		$this->list->c2->set_title(_('Current Version'));
		
		$this->selection = $this->list->view->get_selection();
		//. ... atm there is really nothing to do with it as update.list does not say anything useful.
		$this->selection->connect("changed",'package_view_update');
		$this->selection->set_mode(Gtk::SELECTION_MULTIPLE);

		$this->refresh->connect_simple('clicked','package_list_updates',TRUE);
		$this->update->connect_simple('clicked','package_dialog_install',PKG_UPDATE);
		$this->browse->connect_simple('clicked','package_dialog_update_local_browse',TRUE);
		$this->distupdate->connect_simple('clicked','package_dist_update');

		$this->update->set_sensitive(FALSE);
		
		//. compile view.
//		$this->list->view->remove_column($this->list->c2);	

		$this->row_buttons->pack_end($this->browse,FALSE,FALSE);
		$this->row_buttons->pack_end($this->update,FALSE,FALSE);
		$this->row_buttons->pack_end($this->refresh,FALSE,FALSE);
		$this->row_buttons->pack_start($this->distupdate,FALSE,FALSE);
		
		$this->bottom->pack_start($this->info,TRUE,TRUE);
		$this->bottom->pack_start($this->row_buttons,FALSE,FALSE);
		$this->pane->add1($this->list);
		$this->pane->add2($this->bottom);
		
		
		//. final build.
		$this->pack_start($this->pane,TRUE,TRUE);
	}

}

class packageViewInstall extends GtkVbox {

	function packageViewInstall() {
		parent::__construct();
		
		//. build view widgets.
		$this->pane = new GtkVPaned;
		$this->vb = new GtkVBox;
		$this->buttonbox = new GtkHBox;
		$this->list = new packageListView;
		$this->info = new packageTextView;
		$this->install = new GtkButton(_('Install Selected'));
		$this->browse = new GtkButton(_('Browse...'));
		$this->refresh = new GtkButton(_('Refresh List'));
		$this->dirinstall = new GtkButton(_('Install From Directory'));

		//. configure.
//		$this->list->view->remove_column($this->list->c2);
		$this->pane->set_position(200);
		$this->install->set_sensitive(FALSE);

		$this->selection = $this->list->view->get_selection();
		$this->selection->connect("changed",'package_view_install');
		$this->selection->set_mode(Gtk::SELECTION_MULTIPLE);
		
		$this->refresh->connect_simple('clicked','package_list_installable',TRUE);
		$this->browse->connect_simple('clicked','package_dialog_install_local_browse',TRUE);
		$this->install->connect_simple('clicked','package_dialog_install',PKG_INSTALL);
		$this->dirinstall->connect_simple('clicked','package_dialog_install_directory');
		
		//. compile.
		$this->buttonbox->pack_end($this->browse,FALSE,FALSE);
		$this->buttonbox->pack_end($this->install,FALSE,FALSE);
		$this->buttonbox->pack_end($this->refresh,FALSE,FALSE);
		$this->buttonbox->pack_start($this->dirinstall,FALSE,FALSE);	
		$this->vb->pack_start($this->info,TRUE,TRUE);
		$this->vb->pack_start($this->buttonbox,FALSE,FALSE);
		
		//. final build.
		$this->pane->add1($this->list);
		$this->pane->add2($this->vb);
		$this->pane->pack2($this->buttonbox);
		$this->pack_start($this->pane,TRUE,TRUE);
	}

}

class packageMegaConfirmDialog extends GtkDialog {

	function packageMegaConfirmDialog($title,$icon,$message) {
		parent::__construct();
		
		global $w;
		
		$this->yes = new GtkButton(_('Yes'));
		$this->no = new GtkButton(_('No'));
		$this->cancel = new GtkButton(_('Cancel'));
		$this->message = new GtkLabel;
		$this->status = new packageTextView;
		$this->row_info = new GtkHBox;
		if($icon) {
			$this->icon = katepkg_icon($icon);
		}
	
		$this->set_has_separator(FALSE);
		$this->set_size_request(400,PENDING);
		$this->set_title($title);
		$this->connect_simple('delete-event','katepkg_dialog_block');
		
		$this->set_role('confirm_dialog');
		$this->set_transient_for($w);
		$this->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
		$this->set_modal(TRUE);	
		
		$this->message->set_markup($message);
	
		$this->yes->connect_simple('clicked',array($this,'response'),Gtk::RESPONSE_YES);
		$this->no->connect_simple('clicked',array($this,'response'),Gtk::RESPONSE_NO);
		$this->cancel->connect_simple('clicked',array($this,'response'),Gtk::RESPONSE_CANCEL);
		
		$this->row_info->pack_start($this->icon,FALSE,FALSE);
		$this->row_info->pack_start($this->status,TRUE,TRUE);
		
		$this->vbox->pack_start($this->message,FALSE,FALSE);
		$this->vbox->pack_start($this->row_info,TRUE,TRUE);
		$this->action_area->pack_start($this->cancel,FALSE,FALSE);
		$this->action_area->pack_end($this->no,FALSE,FALSE);
		$this->action_area->pack_end($this->yes,FALSE,FALSE);
		
		$this->status->grab_focus();
	
	}

}

class packageMegaProgressDialog extends GtkDialog {

	function packageMegaProgressDialog($title,$icon,$label) {
		global $w;

		parent::__construct();	

		$this->progress = new GtkProgressBar;
		$this->totalprogress = new GtkProgressBar;
		$this->status = new packageTextView;
		$this->expand = new GtkExpander(_("View Details"));
		$this->icon = katepkg_icon($icon);
		$this->label = new GtkLabel($label);
		$this->cancel = new GtkButton('Done');
		$this->row_label = new GtkHBox;
		$this->row_expand = new GtkHBox;
		$this->special = new GtkButton;
		
		$this->set_has_separator(FALSE);
		$this->set_size_request(400,PENDING);
		$this->set_title($title);
		$this->connect_simple('delete-event','katepkg_dialog_block');
		
		$this->set_role('progress_dialog');
		$this->set_transient_for($w);
		$this->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
		$this->set_modal(TRUE);	
		
		$this->row_label->set_border_width(20);
		
		$this->expand->add($this->status);
		$this->row_expand->pack_start($this->expand,TRUE,TRUE);
		$this->row_label->pack_start($this->icon,FALSE,FALSE);
		$this->row_label->pack_start($this->label,TRUE,TRUE,20);
		$this->vbox->pack_start($this->row_label,FALSE,FALSE);
		$this->vbox->pack_start($this->row_expand,TRUE,TRUE);
		$this->vbox->pack_start($this->progress,FALSE,FALSE);
		$this->vbox->pack_start($this->totalprogress,FALSE,FALSE);
		$this->action_area->pack_start($this->cancel,FALSE,FALSE);
		
		$this->status->grab_focus();	
	}
	
}

function package_quit() {
	package_config_write();
	gtk::main_quit();
	exit(0);
}

function package_run_ui() {
	//. ponder life.
	while(Gdk::events_pending()) {
		Gtk::main_iteration_do(TRUE);
	}
	while(Gtk::events_pending()) {
		Gtk::main_iteration_do(TRUE);
	}
}

function package_config_read() {
	
	global $config;
	global $defaultconfig;
	global $w;

	$arg = func_get_args();
	
	if($arg[0] == TRUE) {

		$config = $defaultconfig;
	
		//. build a config file if it does not exist using my default options.
		if(!file_exists($_SERVER['HOME']."/.katepkgrc")) {
			$fp = fopen($_SERVER['HOME']."/.katepkgrc","w");
			foreach($defaultconfig as $option => $value) {
				fwrite($fp,sprintf("%s %s\n",$option,$value));
			}
			fclose($fp);	
			
		//. read the user configuration options.	
		} else {
			$fp = fopen($_SERVER['HOME']."/.katepkgrc","r");
			while($dump = fgets($fp)) {
				if(preg_match("/^([a-z0-9_]+) (.+)$/i",$dump,$match)) {
					if(isset($defaultconfig[strtolower($match[1])])) {
						if(preg_match("/true/i",$match[2])) { $config[strtolower($match[1])] = (int)TRUE; }
						else if(preg_match("/false/i",$match[2])) { $config[strtolower($match[1])] = (int)FALSE; }
						else { $config[strtolower($match[1])] = $match[2]; }
					}
				}
			}
			fclose($fp);
		}
	
	}

	if($arg[0] == FALSE) {
		//. set any menu check boxes.
		foreach($config as $option => $value) {
			if($w->ui->menuoption[$option] && $value == TRUE) {
				$w->ui->menuoption[$option]->set_active(TRUE);
			}
		}
	}
	
}

function package_config_write()  {

	global $config;
	global $defaultconfig;

	$fp = fopen($_SERVER['HOME']."/.katepkgrc","w");
	foreach($defaultconfig as $option => $value) {
		if($config[$option] === TRUE) { fwrite($fp,sprintf("%s %s\n",$option,"true")); }
		else if($config[$option] === FALSE) { fwrite($fp,sprintf("%s %s\n",$option,"false")); }	
		else { fwrite($fp,sprintf("%s %s\n",$option,$config[$option])); }
	}
	fclose($fp);

}

function package_config_toggle($option) {
	
	global $config;
	global $w;
	
	//print_r($option);
	
	if($config[$option] == TRUE) {
		$config[$option] = FALSE;
//		if($w->ui->menuoption[$option]) {
//			$w->ui->menuoption[$option]->set_active(FALSE);
//		}
		if($w->ui->menuoption[$option]) {
			$w->ui->menuoption[$option]->set_active(FALSE);
		}
	} else if($config[$option] == FALSE) {
		$config[$option] = TRUE;
//		if($w->ui->menuoption[$option]) {
//			$w->ui->menuoption[$option]->set_active(TRUE);
//		}
		if($w->ui->menuoption[$option]) {
			$w->ui->menuoption[$option]->set_active(TRUE);
		}	
	}
	
	package_config_write();
	
}

function package_sourcefile_read() {

	global $updateserver;
	
	//. clear the list.
	$updateserver = array();	
	
	$fp = fopen("/etc/updateservers","r");
	while($dump = fgets($fp)) {
		if(preg_match("/^((?:https?:|ftp:|file:\/)\/\/.+)$/",$dump,$match)) {
			$updateserver[] = $match[1];
		}
		if(preg_match("/^(#\s*(?:https?:|ftp:|file:\/)\/\/.+)$/",$dump,$match)) {
			$updateserver[] = $match[1];
		}
	}
	fclose($fp);

}

function package_fetch_updatelist($ignore) {

	global $updateserver;
	global $pkgremote;
	global $pkgmaster;
	global $w;
	global $config;
	global $selection;
	
	//. clean sweap
	$pkgremote = $pkgmaster = array();

	//. ignore the background window if we want to.	
	if(!$ignore) { $w->ui->set_sensitive(FALSE); }
	
	//. kill selections.
	$w->ui->mode_update->selection->unselect_all();
	$w->ui->mode_install->selection->unselect_all();
	$w->ui->mode_browse->selection->unselect_all();
	$selection = array();
	
	$dialog = new packageMegaProgressDialog(_('Repository Sync'),'sync',_("Syncing with Remote Repositories"));
	$dialog->cancel->connect_simple('clicked',array($dialog,'response'),Gtk::RESPONSE_CANCEL);	
	$dialog->show_all();		
	$dialog->status->grab_focus();	

	katepkg_main_waste();
		
	//. setup the dialog box.
	$dialog->cancel->set_sensitive(FALSE);
	$dialog->cancel->set_label(_('Syncing'));	
	package_run_ui();	

	//. i need to know how many servers there are first.
	$opcount = 0;
	foreach($updateserver as $id => $repo) {
		if(preg_match("/^#/",$repo)) { continue; }
		$opcount++;
	} ++$opcount;
	
	//. begin fetching.
	$dlcount = 0;
	foreach($updateserver as $id => $repo) {
		if(preg_match("/^#/",$repo)) { continue; }
		
		$dialog->status->append_text(sprintf("%s %s... ",_("Scanning"),basename($repo)),TRUE);
		$dialog->progress->set_text(sprintf("%s %s",_("Scanning"),basename($repo)));			
		$dialog->progress->set_fraction(FALSE);
		katepkg_main_waste(10);

		//. fetch the size of the update file.
		$size = katepkg_filesize_remote(sprintf("%s/update.list",$repo));
		
		if(!$size) {
			 $dialog->status->append_text(_("Error").".\n");
		} else {

			//. download the update list.	
			$rp = FALSE;	
			$rp = @fopen(sprintf("%s/update.list",$repo),"r");
			while($dump = fgets($rp)) {
			
				//. im checking deps even though it is not in the format.
				list($pkg,$build,$deps) = explode(" ",str_replace("\n","",$dump));
				preg_match("/(.+)-([^-]+)-([^-]+)-([^-]+)$/",$pkg,$match);
	
				//. ... you know guys... YYYYMMDD can be used in an inequality to determine neweset packages. MMDDYYYY cannot.
				preg_match("/(\d{2})(\d{2})(\d{4})/",$build,$buildmatch);
				
				//. build the repo list.
				$pkgremote[$id][$match[1]] = array(
					'pkg' => $pkg,
					'build' => $build,
					'nicebuild' => $buildmatch[3].$buildmatch[2].$buildmatch[1],
					'deps' => explode('\n',$deps)
				);
				
				//. update the current task progress bar.
				$dialog->progress->set_text(sprintf("%s (%d%%)",basename($repo),((ftell($rp) / $size) * 100)));
				$dialog->progress->set_fraction((ftell($rp) / $size));
				package_run_ui();
				
			} fclose($rp);
			
			//. give status info.
			if($rp) {
				$dialog->status->append_text(sprintf("%s. (%d %s)\n",_("Done"),count($pkgremote[$id]),_("Packages")),TRUE);
			} else {
				$dialog->status->append_text(sprintf("%s.\n",_("Error")),TRUE);
			}

			//. update total task progress bar.
			$dialog->progress->set_fraction(TRUE);
			$dialog->totalprogress->set_fraction((++$dlcount / $opcount));
			
		}
	}
	
	//. update task infos.
	$dialog->progress->set_text(_("Building local package list..."));
	$dialog->progress->set_fraction(TRUE);	
	$dialog->status->append_text(sprintf("%s... ",_("Building local package list")),TRUE);
	
	//. count all the packages before smiting dups and updates.
	foreach($pkgremote as $rid => $list) {
		$pkgcount += count($list);
	}

	//. now i want to build a master lookup array for quick compare against installed packages.
	$loop = 0;
	$upiter = 0;
	foreach($pkgremote as $rid => $list) {
		foreach($list as $pkg => $info) {
			if($pkgmaster[$pkg]) {
				//. if we found a newer build, that is the master.
				if($pkgmaster[$pkg]['nicebuild'] < $info['nicebuild']) {
					$info['rid'] = $rid;
					$pkgmaster[$pkg] = $info;
					unset($pkgmaster[$pkg]['rurl']);
					$pkgmaster[$pkg]['rurl'][0] = preg_replace("/\/$/","",$updateserver[$rid])."/".$info['pkg'].".tgz";
				} else {
					$pkgmaster[$pkg]['rurl'][] = preg_replace("/\/$/","",$updateserver[$rid])."/".$info['pkg'].".tgz";
				}
			} else {
				//. if we have not hit the package yet then add it.
				$info['rid'] = $rid;
				$pkgmaster[$pkg] = $info;
				$pkgmaster[$pkg]['rurl'][0] = preg_replace("/\/$/","",$updateserver[$rid])."/".$info['pkg'].".tgz";
			}
			
			//. update current task progress bar.
			if($pkgcount > 10 && ++$upiter == 10) {
				$dialog->progress->set_fraction((++$loop / $pkgcount));
				$upiter = 0;
			} else {
				++$loop;
			}
			package_run_ui();
		}
	}
	
	//. update the main window view depending on which page we are on.
	switch($w->ui->mode_area->get_current_page()) {
		case(1): {
			package_list_updates();
			break;
		}
		case(2): {
			package_list_installable();
			break;
		}
	}

	package_run_ui();

	//. renable the background gui if we wanted.
	if(!$ignore) { $w->ui->set_sensitive(TRUE); }

	if($config['auto_close_update_dialog'] || $ignore) { 
		$dialog->destroy(); $dialog = FALSE;
	} else {
		//. update task infos.
		$dialog->progress->set_fraction(TRUE);
		$dialog->progress->set_text(_("Done"));
		$dialog->totalprogress->set_fraction((++$dlcount / $opcount));
		$dialog->totalprogress->set_text(_("Done"));
		$dialog->cancel->set_label(_('Done'));	
		$dialog->cancel->set_sensitive(TRUE);
		$dialog->status->append_text(sprintf("\n\n<b>%s.</b>",_("Sync Complete")),TRUE);
		package_run_ui();
		
		$dialog->run();
		
		$dialog->destroy(); $dialog = FALSE;
	}
	
}

function package_page_flip($page) {

	global $w;
	
	$w->ui->mode_area->set_current_page($page);

	//. since set_current_page does not seem to send a switch signal, we'll just
	//. call our function ourself :D	
	package_page_select(FALSE,FALSE,$page);

}

function package_page_select($w1,$w2,$page) {

	global $w;
	global $config;
	global $selected;	
	
	//. page change signal gets emmited before the window is even created and i forget why so
	//. we might as well stop now if there is no bloody window.
	if(!$w->basetitle) { return; }

	//. no matter what, switching tabs is switching modes. therefore all selections must be dropped.
	$w->ui->mode_browse->selection->unselect_all();
	$w->ui->mode_update->selection->unselect_all();
	$w->ui->mode_install->selection->unselect_all();
	$selected = FALSE;
	$w->ui->mode_update->update->set_sensitive(FALSE);
	$w->ui->mode_install->install->set_sensitive(FALSE);
	
	switch($page) {
		case(0): { //. the browse tab.
//			package_list_installed();
			package_list_installed(TRUE);
			break;
		}
		case(1): { //. the update tab.
			package_list_updates();
			break;
		}
		case(2): { //. the install tab.
			package_list_installable();
			break;
		}
	}
	
}

function package_list_installed() {
	
	global $w;
	global $pkglist;
	global $selected;
	global $init;
	
	if($init) {
		gtk::idle_remove($init);
		$init = FALSE;
	}

	$arg = func_get_args();

	//. setting the base statusbar help for this tab.
	$w->ui->status_text(_("View packages installed on your system."));
	
	if($arg[0] == TRUE) {
		$w->update_title(_("Browse Installed Packages"));
		return FALSE;
	}
	
	$pkglist = array();
	
	//. change things about the window to let the user know it could be working for a bit.
	$w->update_title(sprintf("%s (%s)",_("Browse Installed Packages"),_("checking system")));
	$w->ui->mode_browse->refresh->set_sensitive(FALSE);
	$w->ui->mode_browse->info->clear_text();
	
	//. i want most of the window to be disabled while we perform this scan.
	$w->ui->set_sensitive(FALSE);
	
	//. set to our empty model so that the refreshing of the treeview does not lag the ui.
	$w->ui->mode_browse->list->view->set_model($w->ui->mode_browse->list->empty);
	$w->ui->mode_browse->list->store->clear();
	
	//. reset the progress metre.
	$w->ui->progress->set_fraction(0);
	$w->ui->progress->set_text(sprintf("%s...",_("Scanning")));
	while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }
	
	$directory = scandir(LOGROOT);
	$count = count($directory);

	$a = 0;
	//. scan the system packages.
	foreach($directory as $dump) {
		++$a;
		if(is_file(LOGROOT."/".$dump)) {
			if(preg_match("/(.+)-([^\-]+)-([^\-]+)-([^\-]+)$/",$dump,$match)) {
				$pkglist[$match[1]][0] = TRUE;
				$pkglist[$match[1]] = package_read_info($dump);
				
				if($pkglist[$match[1]]['desc'][LANGUAGE]) {
					if(!$pkglist[$match[1]]['desc'][LANGUAGE][0]) { $pkglist[$match[1]]['desc'][LANGUAGE][0] = ""; }
					$w->ui->mode_browse->list->store->append(array($dump,$pkglist[$match[1]]['desc'][LANGUAGE][0]));
				} else {
					if(!$pkglist[$match[1]]['desc']['en_US'][0]) { $pkglist[$match[1]]['desc']['en_US'][0] = ""; }
					$w->ui->mode_browse->list->store->append(array($dump,$pkglist[$match[1]]['desc']['en_US'][0]));
				}

				/*
				$r = floor(($a / $count) * 100);
				if($r == 25 || $r == 50 || $r == 75) {
					$w->ui->progress->set_fraction($r / 100);
					while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }
				} */
	
			}
		}
	}

	while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }
	
	//. random pokes.
	//. it seems the first read back is the killer in PHP... pretty sure it is not my fault... so i am going try and
	//. get that over with before the user can do anything and see if that helps the user think.
	foreach($pkglist as $pkg => $data) {
//		printf("preloading %s\n",$pkg);
		package_read_info($data[$pkg],TRUE);
		break;
	}
	
	//. with the work finished, swap to the full model.
	$w->ui->mode_browse->list->view->set_model($w->ui->mode_browse->list->store);

	//. disable any buttons that require selected packages.
	$w->ui->mode_browse->uninstall->set_sensitive(FALSE);
	$w->ui->mode_browse->info->clear_text();
	
	//. click this once to sort by alpha.
	if($w->ui->mode_browse->list->clicked_once == FALSE) {
		$w->ui->mode_browse->list->c1->clicked();
		$w->ui->mode_browse->list->clicked_once = TRUE;
	}

	//. and fill the progress to indicate the task is complete.
	$w->ui->progress->set_fraction(1);
	$w->ui->progress->set_text(" ");

	//. drop what might be selected.
	$selected = FALSE;
	$w->ui->mode_browse->selection->unselect_all();
	$w->ui->mode_update->selection->unselect_all();
	$w->ui->mode_update->update->set_sensitive(FALSE);
	$w->ui->mode_install->selection->unselect_all();
	$w->ui->mode_install->install->set_sensitive(FALSE);
	
	//. give back control of the window.
	$w->ui->set_sensitive(TRUE);	
	
	//. change things about the window to tell the user it is ready for use.
	$w->update_title(sprintf("%s",_("Browse Installed Packages")));
	$w->ui->mode_browse->refresh->set_sensitive(TRUE);
	
}

function package_read_info($package) {

	global $pkglist;
	
	//. the second argument could be TRUE, or omitted. TRUE means, give me extended info.
	$arg = func_get_args();
	
	//. set description dictation mode off.
	$dict_desc = FALSE;	
	
	//. find relevent information from the package name itself.
	preg_match("/(.+)-([^-]+)-([^-]+)-([^-]+)$/",$package,$match);
	$output['pkg'] = $match[0];
	$output['package'] = $match[1];
	$output['ver'] = $match[2];
	$output['arch'] = $match[3];
	$output['build'] = $match[4];
	
	$output[0] = TRUE;	
	
	//. read the local build file.
	if(file_exists(BUILDROOT."/".$package)) {
		preg_match("/(\d{2})(\d{2})(\d{4})/",file_get_contents(BUILDROOT."/".$package),$buildmatch);
		$output['build'] = $buildmatch[1].$buildmatch[2].$buildmatch[3 ];	
		$output['nicebuild'] = $buildmatch[3].$buildmatch[2].$buildmatch[1];
	} else {
		$output['build'] = 99999999; //. lool.
		$output['nicebuild'] = 99999999;
	}
	

	//. read the package log.
	$dump = file_get_contents(LOGROOT."/".$package);
	preg_match_all("/^(?:(?:".$match[1].": ?([^\n]*))|(?:LANGUAGE: *([^\n]+)))/ms",$dump,$dumpmatch);
	
	$dict_lang = "en_US"; $a = 0;
	for($a = 0; $a < count($dumpmatch[0]); $a++) {
		if($dumpmatch[2][$a]) {
			$dict_lang = $dumpmatch[2][$a];
		} else {
			$output['desc'][$dict_lang][] = iconv("ISO-8859-2","UTF-8",$dumpmatch[1][$a]);
//			$output['desc'][$dict_lang][] = iconv(mb_dectect_decoding($dumpmatch[1][$a]),"UTF-8",$dumpmatch[1][$a]);
		}
	}
	
	preg_match('/^COMPRESSED PACKAGE SIZE: *([^\n]+)$/ms',$dump,$dumpmatch);
	$output['pkgsize'] = iconv("ISO-8859-2","UTF-8",$dumpmatch[1]);
	preg_match('/^UNCOMPRESSED PACKAGE SIZE: *([^\n]+)$/ms',$dump,$dumpmatch);
	$output['size'] = iconv("ISO-8859-2","UTF-8",$dumpmatch[1]);
	

	//. if we wanted extended information now is the time to do it.
	if($arg[1] == TRUE) {

		//. find what this depends on.
		if(file_exists(DEPSROOT."/".$package)) { //. a case where a package did not have a deps file er bad build. :p
			$deps = explode("\n",file_get_contents(DEPSROOT."/".$package));
		} else {
			$deps = array();
		}
		
		for($a = 0; $a < count($deps); $a++) {
			if(!$deps[$a]) {
				unset($deps[$a]); //. remove blank lines.
			}
		}

		//. final dependency array.
		$output['deps'] = $deps;	
		
		//. find what depends on this.
		$pp = popen(sprintf('grep -rHP "^%s$" /var/log/deps/ | sed "s/.*\/\([^\/\]\+\):.\+$/\1/"',$output['package']),"r");
		while($dump = fgets($pp)) {
			$dependents[] = str_replace("\n","",$dump);
		}
		fclose($pp);
		
		//. final dependents array.
		$output['dependents'] = $dependents;
		
	}
	
	//. return the package data.
	$pkglist[$match[1]] = $output;
	
	return $output;

}

function package_view_details() {
	
	global $w;	
	global $pkglist;
	global $selected;
	
	//. the user has selected something from the tree view therefore we must note what was selected and
	//. display information about it.

	//. clear old selection
	$selected = array();
	
	//. pull an array of iterators from the list.
	list($null, $iter) = $w->ui->mode_browse->selection->get_selected_rows();
	if(!$iter[0]) { $iter = array(); }

	//. and get me text data i can work with from the list from the iterator array.
	foreach($iter as $row) {
		$selected[] = $w->ui->mode_browse->list->store->get_value(
			$w->ui->mode_browse->list->store->get_iter($row),
			0
		);
	}

	//. if only one package is selected then the info is displayed.
	if(count($selected) == 1) {
		$info = package_read_info($selected[0],TRUE);

		//. format the description.
		if($info['desc'][LANGUAGE]) {
			$description = $info['desc'][LANGUAGE];		
		} else {
			$description = $info['desc']['en_US'];
		}		
		
		$quickinfo = $description[0]; unset($description[0]);
		
		if(count($description) > 1) {
			$description = implode("\n",$description); //. merge
		}

		$description = preg_replace("/(?:^ {2,})|(?:^$)/ms","",$description);
		$description = preg_replace("/\n/ms"," ",$description);
//		$description = preg_replace("/(?:^ {2,})|\n$/ms","",$description);
//		$description = preg_replace("/\n\n/ms","",$description);
//		$description = preg_replace("/\n/ms","\n\n",$description);
//		$description = preg_replace("/^ /ms","",$description);
//		$description = preg_replace("/\n$/","",$description);

		//. swap buffer view for render snappiness.
		$w->ui->mode_browse->info->view->set_buffer($w->ui->mode_browse->info->empty);

		//. show info
		$w->ui->mode_browse->info->clear_text();
		$w->ui->mode_browse->info->append_text(sprintf(
			"<b>%s</b>\n%s\n\n<b>%s:</b> %s.\n<b>%s:</b> %s.\n\n",
			$selected[0],
			(($quickinfo)?($quickinfo):($info['pkg'])),
			_("Package Size"),
			$info['pkgsize'],
			_("Size On Disk"),
			$info['size']
		));
		

		//. show description.
		$w->ui->mode_browse->info->append_text(sprintf("%s\n\n",$description));

		//. list dependancies.
		if(count($info['deps']) > 0) {
			$w->ui->mode_browse->info->append_text(sprintf("<b>%s:</b>\n",_("Dependencies")));
			foreach($info['deps'] as $dep) {
				$w->ui->mode_browse->info->append_text(sprintf("\t%s\n",$dep));	
			}			
		} else {
			$w->ui->mode_browse->info->append_text(sprintf("<b>%s:</b> %s.\n",_("Dependencies"),_("None")));
		}		
		
		//. list dependents.
		if(count($info['dependents']) > 0) {
			$w->ui->mode_browse->info->append_text(sprintf("\n<b>%s:</b>\n",_("Dependent Packages")));
			foreach($info['dependents'] as $dep) {
				$w->ui->mode_browse->info->append_text(sprintf("\t%s\n",$dep));	
			}	
		} else {
			$w->ui->mode_browse->info->append_text(sprintf("\n<b>%s:</b> %s.\n",_("Dependent Packages"),_("None")));
		}
		
		$w->ui->mode_browse->uninstall->set_sensitive(TRUE);
		
		//. swap buffer view for render snappiness.
		$w->ui->mode_browse->info->view->set_buffer($w->ui->mode_browse->info->buffer);
		
	//. if multiple packages are selected, then cumilative info is shown.
	} else if(count($selected) > 1) {
		$w->ui->mode_browse->info->clear_text();
		$w->ui->mode_browse->info->append_text(sprintf(
			"<b>%s: </b> %d.\n",
			_("Total Packages"),
			count($selected)
		));
		
		foreach($selected as $pkg) {
			$info = package_read_info($pkg,TRUE);
			$totalpkgsize += ((int)$info['pkgsize']); //. this drops the non-int chars (the K) while retaining the important stuff.
			$totalsize += ((int)$info['size']);
			$totallist = sprintf("%s\t%s\n",$totallist,$pkg);
		}
		
		$w->ui->mode_browse->info->append_text(sprintf(
			"<b>%s:</b> %d K.\n<b>%s:</b> %d K.\n<b>%s:</b>\n%s",
			_("Total Packages Size"),
			$totalpkgsize,
			_("Total Size On Disk"),
			$totalsize,
			_("Selected Packages"),
			$totallist
		));
		
		$w->ui->mode_browse->uninstall->set_sensitive(TRUE);
		
	//. if none are selected, then the info area is cleared.
	} else {
		$w->ui->mode_browse->uninstall->set_sensitive(FALSE);
		$w->ui->mode_browse->info->clear_text();	
	}
}

// ////////////////////////////////////////////////////////////
//. functions which relate to the source file editor.                        //
                               ///////////////////////////////////////////// //

function katepkg_srcedit_dialog() {

	global $w;
	global $dialog;
	global $updateserver,$config;

	$templist = $updateserver;

	$w->ui->set_sensitive(FALSE);

	$dialog = new GtkDialog;
	$dialog->scroller = new GtkScrolledWindow;
	$dialog->set_transient_for($w);
	$dialog->set_position(Gtk::WIN_POS_CENTER);
	$dialog->set_modal(TRUE);
	$dialog->set_title(_("Repository List Editor"));
	$dialog->set_resizable(TRUE);
	
	$dialog->edit_area = new GtkVBox;
	$dialog->set_size_request(-1,300);
	$dialog->scroller->set_policy(Gtk::POLICY_NEVER,Gtk::POLICY_AUTOMATIC);
	$dialog->scroller->add_with_viewport($dialog->edit_area);
	$dialog->vbox->pack_start($dialog->scroller);
	
	$dialog->addbox = new GtkHBox();
	$dialog->addentry = new GtkEntry;
	$dialog->add = new GtkButton;
	$dialog->add->connect_simple('clicked',array($dialog,'response'),Gtk::RESPONSE_ACCEPT);
	$dialog->add->set_image(katepkg_icon('add'));
	$dialog->add->set_focus_on_click(FALSE);
	$dialog->vbox->pack_start($dialog->addbox,FALSE,FALSE);
	
	$dialog->addbox->pack_start($dialog->add,FALSE,FALSE);
	$dialog->addbox->pack_start($dialog->addentry,TRUE,TRUE);

	$dialog->hbox = array();

	$dialog->connect_simple('delete-event',array($dialog,'response'),Gtk::RESPONSE_CANCEL);

	$dialog->add_button(_("Cancel"),Gtk::RESPONSE_CANCEL);
	$dialog->add_button(_("Save"),Gtk::RESPONSE_APPLY);

	$run = TRUE;
	while($run == TRUE) {
	
		foreach($dialog->hbox as $id => $widget) {
			$dialog->hbox[$id]->hide_all();
			$dialog->del[$id]->destroy();
			$dialog->com[$id]->destroy();
			$dialog->label[$id]->destroy();
			$dialog->hbox[$id]->destroy();
		}

		foreach($templist as $id => $url) {
			$dialog->hbox[$id] = new GtkHBox;
			$dialog->del[$id] = new GtkButton;
			$dialog->com[$id] = new GtkToggleButton;
			$dialog->label[$id] = new GtkEntry($url);

			$dialog->label[$id]->set_size_request(300,PENDING);
			$dialog->label[$id]->connect_simple('focus-out-event',array($dialog,'response'),Gtk::RESPONSE_HELP);
			
			$dialog->del[$id]->connect_simple('clicked',array($dialog,'response'),$id);
			$dialog->del[$id]->set_image(katepkg_icon('remove'));
			$dialog->del[$id]->set_focus_on_click(FALSE);

			$dialog->com[$id]->connect_simple('clicked',array($dialog,'response'),(($id + 1) * 1000));
			$dialog->com[$id]->set_focus_on_click(FALSE);
			
			if(preg_match("/^#/",$url)) {
				$dialog->com[$id]->set_image(katepkg_icon('block'));			
				$dialog->com[$id]->set_active(TRUE);
				$dialog->label[$id]->set_text(preg_replace("/^#\s*/","",$url));
				$dialog->label[$id]->set_sensitive(FALSE);
			} else {
				$dialog->com[$id]->set_image(katepkg_icon('active'));
			}

			$dialog->hbox[$id]->pack_start($dialog->del[$id],FALSE,FALSE);
			$dialog->hbox[$id]->pack_start($dialog->com[$id],FALSE,FALSE);
			$dialog->hbox[$id]->pack_start($dialog->label[$id],TRUE,TRUE);
			
			$dialog->edit_area->pack_start($dialog->hbox[$id],TRUE,TRUE);

		}

//. disabled to create a streching effect that users seemed to like.
//		$dialog->resize(TRUE,TRUE);

		$dialog->show_all();
	
		$click = $dialog->run();
		
		switch($click) {
			case(Gtk::RESPONSE_CANCEL): {
				$run = FALSE;
				break;
			}
			case(Gtk::RESPONSE_ACCEPT): {
			
				katepkg_srcedit_read($templist);
					
				if($new = $dialog->addentry->get_text()) {
					if(!in_array($new,$templist)) {
						$templist[] = $new;
					}
					$dialog->addentry->set_text("");
				}
				
				break;
			}
			case(Gtk::RESPONSE_APPLY): {

				katepkg_srcedit_read($templist);
						
				katepkg_srcedit_save($templist);
				$run = FALSE;
				break;
			}
			case(Gtk::RESPONSE_HELP): {
				katepkg_srcedit_read($templist);
				break;			
			}
			default: {
				
				if($config['debug']) { printf("click %d (id %d)\n",$click,(($click / 1000) - 1)); }				

				katepkg_srcedit_read($templist);
								
				if($click >= 1000) {
					$rid = (($click / 1000) - 1);
					if(preg_match("/^#/",$templist[$rid])) {
						$templist[$rid] = preg_replace("/^#\s*/","",$templist[$rid]);
					} else {
						$templist[$rid] = "# " . $templist[$rid];
					}
				} else {
					unset($templist[$click]);
				}
			
				break;
			}
		}		
		
	}

	$dialog->hide_all();
	
	if($click == Gtk::RESPONSE_APPLY) {
		package_sourcefile_read();
//		package_fetch_updatelist();
	}	

	$dialog->destroy();
	
	$w->ui->set_sensitive(TRUE);
}

function katepkg_srcedit_read(&$templist) {

	global $dialog;
	global $config;
	
	$templist2 = array();
	foreach($templist as $id => $widget) {
	
		if(preg_match("/^#/",$templist[$id])) {
			$templist2[] = "# " . $dialog->label[$id]->get_text();
		} else {
			$templist2[] = $dialog->label[$id]->get_text();
		}
	}
	
	$templist = $templist2;
	
	if($config['debug']) {
		print_r($templist);
	}

}

function katepkg_srcedit_save($list) {

	global $w;
	global $updateserver;
	
	$updateserver = $list;
	
	$fp = fopen("/etc/updateservers","w");
	fwrite($fp,sprintf("# %s\n# %s (%s)\n\n",_("KateOS Repository Listing"),_("Generated by KatePKG"),date("Y/m/d H:i:s")));
	foreach($list as $repo) {
		fwrite($fp,sprintf("%s\n",$repo));
	}
	fwrite($fp,"\n");
	fclose($fp);
	
	$w->ui->status_text_timeout(_("Repository List Saved"));

}

// ////////////////////////////////////////////////////////////
//. generic functions                                                        //
                               ///////////////////////////////////////////// //

function katepkg_array_rotate(&$input) {

	$input[] = $input[0];
	unset($input[0]);

	$iter = 0;
	foreach($input as $element) {
		$newinput[$iter++] = $element;
	}

	$input = $newinput;

}
                               
function katepkg_dialog_block() {
	//. this just returns true to stop things like
	//. delete-events when i want.
	return TRUE;
}

function katepkg_main_waste() {

	$arg = func_get_args();
	if(!$arg[0]) { $arg[0] = 50; }

	for($a = 0; $a < $arg[0]; $a++) {
		//. i'm wasting my time, i'm wasting my time.
		Gtk::main_iteration_do(FALSE);
		usleep(250);
//		echo(":p\n");
	}
	
	return FALSE;
}

function katepkg_dprintf() {

	global $config;
	
	if(!$config['debug']) { return FALSE; }

	$arg = func_get_args();

	$format = $arg[0];
	unset($arg[0]);

	return vprintf($format,$arg);
}

function katepkg_window_resize() {

	global $config;
	global $w;
	
	if(!$w->active) { return FALSE; }
	
	$size = $w->get_size();
	
	switch($w->ui->mode_area->get_current_page()) {
		case(0): { $pos = $w->ui->mode_browse->pane->get_position(); break; }
		case(1): { $pos = $w->ui->mode_update->pane->get_position(); break; }
		case(2): { $pos = $w->ui->mode_install->pane->get_position(); break; }
	}
	
	if($size[0] != $config['katepkg_width']) { $config['katepkg_width'] = $size[0]; }
	if($size[1] != $config['katepkg_height']) { $config['katepkg_height'] = $size[1]; }
	if($pos != $config['katepkg_barpos']) {
		$config['katepkg_barpos'] = $pos;
		$w->ui->mode_browse->pane->set_position($pos);
		$w->ui->mode_update->pane->set_position($pos);
		$w->ui->mode_install->pane->set_position($pos);
	}

	return FALSE;

}

function katepkg_open_download_directory() {
	global $w;
	if($w->ui->opencmd) {
		system(sprintf($w->ui->opencmd,DLROOT));
	}
}

function katepkg_icon($icon) {
	return GtkImage::new_from_file(EXECROOT."/share/pixmaps/katepkg/".$icon.".png");
}

function katepkg_dialog_about() {

	global $w;

	$w->ui->set_sensitive(FALSE);	
	
	$dialog = new GtkDialog;
	$dialog->icon = katepkg_icon('katepkg-100');
	$dialog->close = new GtkButton(_('Close'));
	$dialog->message = new GtkLabel;
	$dialog->status = new packageTextView;
	$dialog->row_info = new GtkHBox;

	$dialog->set_transient_for($w);
	$dialog->set_position(Gtk::WIN_POS_CENTER);
	$dialog->set_title(_('About KatePKG'));
	$dialog->set_has_separator(FALSE);
//	$dialog->set_resizable(FALSE);
	$dialog->set_role('about_dialog');
	
	$dialog->close->connect_simple('clicked',array($dialog,'response'),Gtk::RESPONSE_CANCEL);
//	$dialog->set_size_request(300,300);
	$dialog->status->set_size_request(-1,150);
	
	$dialog->message->set_markup(sprintf("<span size='xx-large' weight='bold'>KatePKG v%s</span>\n%s: %s (%s)",VERSION,_('Release'),NAME,date("Y/m/d",DATE)));
	
	$dialog->status->append_text(sprintf(
		"%s.

%s

<b>%s:</b> Bob Majdak Jr <bob@kateos.org>
<b>%s:</b> http://wikidoc.kateos.org
<b>%s:</b>
	Kamil Stachowski (%s)
	Christian Herenz (%s)
	
%s:

KatePKG v%s %s %s
Locale: %s
%s %s %s
PHP: %s, PHP-GTK %s
GTK+ %s
		",
		_("KatePKG is a GTK2 tool for managing the software packages on KateOS"),
		_("This program is released under the GNU GPL."),
		_("Authour"),
		_("Documentation"),
		_("Translations"),
		_("Polish"),
		_("German"),
		_("Please report all bugs to the authours email above, and include the following information in addition to the error and what you did to cause it"),
		VERSION, NAME, DATE,
		LOCALE,
		php_uname('s'), php_uname('r'), php_uname('m'),
		phpversion(), phpversion('php-gtk'),
		GTK2
		
	));
	
	$dialog->row_info->pack_start($dialog->icon,FALSE,FALSE);
	$dialog->row_info->pack_start($dialog->message,TRUE,TRUE);		
	
	$dialog->vbox->pack_start($dialog->row_info,FALSE,FALSE);
	$dialog->vbox->pack_start($dialog->status,TRUE,TRUE);
	$dialog->action_area->pack_start($dialog->close);
	
	$dialog->show_all();
	
	$dialog->run();
	
	$dialog->destroy(); $dialog = FALSE;
	
	$w->ui->set_sensitive(TRUE);
	
	return TRUE;

}

?>